@@ -18,6 +18,7 @@ from typing import (
     List,
     Mapping,
     Optional,
+    Set,
     Type,
     Union,
 )
@@ -28,6 +29,7 @@ from kiara.defaults import KIARA_DB_MIGRATIONS_CONFIG, KIARA_DB_MIGRATIONS_FOLDE
 from kiara.interfaces import get_console
 from kiara.interfaces.python_api import StoreValueResult, StoreValuesResult
 from kiara.models import KiaraModel
+from kiara.models.context import ContextSummary
 from kiara.models.info import InfoModelGroup, ItemInfo, KiaraModelClassesInfo
 from kiara.models.module import KiaraModuleTypeInfo, ModuleTypeClassesInfo
 from kiara.models.module.manifest import Manifest
@@ -94,15 +96,28 @@ class Kiara(object):
     how something was created, and in which environment.
     """
 
-    _instance = None
+    _instances: Dict[str, "Kiara"] = {}
+    _instance_kiara_config = KiaraConfig()
 
     @classmethod
-    def instance(cls) -> "Kiara":
+    def instance(cls, context_name: Optional[str] = None) -> "Kiara":
         """The default *kiara* context. In most cases, it's recommended you create and manage your own, though."""
 
-        if cls._instance is None:
-            cls._instance = Kiara()
-        return cls._instance
+        # TODO: make this thread-safe
+        if context_name is None:
+            _context_name = os.environ.get("KIARA_CONTEXT", None)
+        else:
+            _context_name = context_name
+
+        if not _context_name:
+            _context_name = cls._instance_kiara_config.default_context
+
+        if _context_name in cls._instances.keys():
+            return cls._instances[_context_name]
+
+        kiara = cls._instance_kiara_config.create_context(context=context_name)
+        cls._instances[_context_name] = kiara
+        return kiara
 
     def __init__(self, config: Optional[KiaraContextConfig] = None):
 
@@ -166,6 +181,7 @@ class Kiara(object):
             config_cls = archive_cls._config_cls
             archive_config = config_cls(**archive.config)
             archive_obj = archive_cls(archive_id=archive.archive_uuid, config=archive_config)  # type: ignore
+
             for supported_type in archive_obj.supported_item_types():
                 if supported_type == "data":
                     self.data_registry.register_data_archive(
@@ -177,6 +193,9 @@ class Kiara(object):
                 if supported_type == "alias":
                     self.alias_registry.register_archive(archive_obj, alias=archive_alias)  # type: ignore
 
+                if supported_type == "destiny":
+                    self.destiny_registry.register_destiny_archive(archive_obj, alias=archive_alias)  # type: ignore
+
     def _run_alembic_migrations(self):
         script_location = os.path.abspath(KIARA_DB_MIGRATIONS_FOLDER)
         dsn = self._config.db_url
@@ -378,40 +397,24 @@ class Kiara(object):
 
         return StoreValuesResult.construct(__root__=stored)
 
-    # def run(self, module_or_operation: str, module_config: Mapping[str, Any] = None):
-    #
-    #     if isinstance(module_or_operation, str):
-    #         if module_or_operation in self.operation_registry.operation_ids:
-    #
-    #             operation = self.operation_registry.get_operation(module_or_operation)
-    #             if module_config:
-    #                 print(
-    #                     f"Specified run target '{module_or_operation}' is an operation, additional module configuration is not allowed."
-    #                 )
-    #
-    #     elif module_or_operation in self.module_type_names:
-    #
-    #         if module_config is None:
-    #             module_config = {}
-    #         manifest = Manifest(
-    #             module_type=module_or_operation, module_config=module_config
-    #         )
-    #
-    #         module = self.create_module(manifest=manifest)
-    #         operation = Operation.create_from_module(module)
-    #
-    #     elif os.path.isfile(module_or_operation):
-    #         raise NotImplementedError()
-    #         # module_name = kiara_obj.register_pipeline_description(
-    #         #     module_or_operation, raise_exception=True
-    #         # )
-    #     else:
-    #         merged = list(self.module_type_names)
-    #         merged.extend(self.operation_registry.operation_ids)
-    #         raise NoSuchExecutionTargetException(
-    #             msg=f"Invalid run target name '[i]{module_or_operation}[/i]'. Must be a path to a pipeline file, or one of the available modules/operations.",
-    #             available_targets=sorted(merged),
-    #         )
+    def create_context_summary(self) -> ContextSummary:
+        return ContextSummary.create_from_context(kiara=self)
+
+    def get_all_archives(self) -> Dict[KiaraArchive, Set[str]]:
+
+        result: Dict[KiaraArchive, Set[str]] = {}
+
+        archive: KiaraArchive
+        for alias, archive in self.data_registry.data_archives.items():
+            result.setdefault(archive, set()).add(alias)
+        for alias, archive in self.alias_registry.alias_archives.items():
+            result.setdefault(archive, set()).add(alias)
+        for alias, archive in self.destiny_registry.destiny_archives.items():
+            result.setdefault(archive, set()).add(alias)
+        for alias, archive in self.job_registry.job_archives.items():
+            result.setdefault(archive, set()).add(alias)
+
+        return result
 
 
 class KiaraContextInfo(KiaraModel):
@@ -526,3 +529,48 @@ class KiaraContextInfo(KiaraModel):
             result["operations"] = self.operations
 
         return result
+
+
+def delete_context(kiara_config: KiaraConfig, context_name: str):
+
+    kiara_context_config = kiara_config.get_context_config(context_name=context_name)
+    kiara = Kiara(config=kiara_context_config)
+
+    data_archives = kiara.data_registry.data_archives.values()
+    alias_archives = kiara.alias_registry.alias_archives.values()
+    job_archives = kiara.job_registry.job_archives.values()
+    destiny_archives = kiara.destiny_registry.destiny_archives.values()
+
+    clashes: Dict[str, List[KiaraArchive]] = {}
+    for context_name, context_config in kiara_config.context_configs.items():
+        k = Kiara(config=context_config)
+        for da in k.data_registry.data_archives.values():
+            if da in data_archives:
+                clashes.setdefault("data", []).append(da)
+        for aa in k.alias_registry.alias_archives.values():
+            if aa in alias_archives:
+                clashes.setdefault("alias", []).append(aa)
+        for ja in k.job_registry.job_archives.values():
+            if ja in job_archives:
+                clashes.setdefault("job", []).append(ja)
+        for dea in k.destiny_registry.destiny_archives.values():
+            if dea in destiny_archives:
+                clashes.setdefault("destiny", []).append(dea)
+
+    if clashes:
+        # TODO: only delete non-clash archives and don't throw exception
+        raise Exception(
+            f"Can't delete context '{context_name}', some archives are used in other contexts: {clashes}"
+        )
+
+    for da in data_archives:
+        da.delete_archive(archive_id=da.archive_id)
+
+    for aa in alias_archives:
+        aa.delete_archive(archive_id=aa.archive_id)
+
+    for ja in job_archives:
+        ja.delete_archive(archive_id=ja.archive_id)
+
+    for dea in destiny_archives:
+        dea.delete_archive(archive_id=dea.archive_id)
