@@ -25,28 +25,55 @@ be discouraged, since this might not be trivial and there are quite a few things
 """
 import abc
 import copy
+import orjson
+import structlog
 import uuid
 from deepdiff import DeepHash
 from pydantic import BaseModel, Extra, Field, PrivateAttr, ValidationError
 from rich import box
 from rich.console import Console, ConsoleOptions, RenderResult
+from rich.rule import Rule
+from rich.syntax import Syntax
 from rich.table import Table
-from typing import Any, Generic, Mapping, Optional, Tuple, Type, TypeVar, Union
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Generic,
+    Mapping,
+    Optional,
+    Tuple,
+    Type,
+    TypeVar,
+    Union,
+)
 
-from kiara.defaults import KIARA_HASH_FUNCTION, SpecialValue
+from kiara.defaults import (
+    INVALID_HASH_MARKER,
+    INVALID_SIZE_MARKER,
+    KIARA_HASH_FUNCTION,
+    NO_SERIALIZATION_MARKER,
+    SpecialValue,
+)
 from kiara.exceptions import KiaraValueException, ValueTypeConfigException
 from kiara.models.python_class import PythonClass
 from kiara.models.values import ValueStatus
-from kiara.models.values.value import Value, ValuePedigree
 from kiara.models.values.value_schema import ValueSchema
 
-# def get_type_name(obj: Any):
-#     """Utility function to get a pretty string from the class of an object."""
 #
 #     if obj.__class__.__module__ == "builtins":
 #         return obj.__class__.__name__
 #     else:
 #         return f"{obj.__class__.__module__}.{obj.__class__.__name__}"
+from kiara.utils import orjson_dumps
+
+if TYPE_CHECKING:
+    from kiara.models.values.value import SerializedData, Value, ValuePedigree
+
+
+logger = structlog.getLogger()
+
+# def get_type_name(obj: Any):
+#     """Utility function to get a pretty string from the class of an object."""
 
 
 class DataTypeConfig(BaseModel):
@@ -56,6 +83,11 @@ class DataTypeConfig(BaseModel):
     a ``DataType`` is not configurable, unless the ``_config_cls`` class attribute points to a sub-class of this class.
     """
 
+    class Config:
+        json_loads = orjson.loads
+        json_dumps = orjson_dumps
+        extra = Extra.forbid
+
     @classmethod
     def requires_config(cls) -> bool:
         """Return whether this class can be used as-is, or requires configuration before an instance can be created."""
@@ -67,10 +99,6 @@ class DataTypeConfig(BaseModel):
 
     _config_hash: Optional[int] = PrivateAttr(default=None)
 
-    class Config:
-        extra = Extra.forbid
-        # allow_mutation = False
-
     def get(self, key: str) -> Any:
         """Get the value for the specified configuation key."""
 
@@ -148,6 +176,10 @@ class DataType(abc.ABC, Generic[TYPE_PYTHON_CLS, TYPE_CONFIG_CLS]):
      data, say a single column of a table. Or when a frontend needs to display/visualize the data.
     """
 
+    @classmethod
+    def retrieve_available_type_profiles(cls) -> Mapping[str, Mapping[str, Any]]:
+        return {}
+
     @classmethod
     @abc.abstractmethod
     def python_class(cls) -> Type[TYPE_PYTHON_CLS]:
@@ -214,13 +246,82 @@ class DataType(abc.ABC, Generic[TYPE_PYTHON_CLS, TYPE_CONFIG_CLS]):
     # def is_immutable(self) -> bool:
     #     pass
 
-    @abc.abstractmethod
-    def calculate_hash(self, data: TYPE_PYTHON_CLS) -> int:
+    def calculate_hash(self, data: "SerializedData") -> str:
         """Calculate the hash of the value."""
 
-    @abc.abstractmethod
-    def calculate_size(self, data: TYPE_PYTHON_CLS) -> int:
-        pass
+        return data.instance_id
+
+    def calculate_size(self, data: "SerializedData") -> int:
+        """Calculate the size of the value."""
+
+        return data.data_size
+
+    def serialize_as_json(self, data: Any) -> "SerializedData":
+
+        _data = {"data": {"type": "inline-json", "inline_data": data, "codec": "json"}}
+
+        serialized_data = {
+            "data_type": self.data_type_name,
+            "data_type_config": self.type_config.dict(),
+            "data": _data,
+            "serialization_profile": "json",
+            "metadata": {
+                "environment": {},
+                "deserialize": {
+                    "python_object": {
+                        "module_type": "deserialize.from_json",
+                        "module_config": {"result_path": "data"},
+                    }
+                },
+            },
+        }
+        from kiara.models.values.value import SerializationResult
+
+        serialized = SerializationResult(**serialized_data)
+        return serialized
+
+    def serialize(self, data: TYPE_PYTHON_CLS) -> Union[None, str, "SerializedData"]:
+
+        logger.debug(
+            "ignore.serialize_request",
+            data_type=self.data_type_name,
+            reason="no 'serialize' method imnplemented",
+        )
+        return NO_SERIALIZATION_MARKER
+        # raise NotImplementedError(f"Data type '{self.data_type_name}' does not support serialization.")
+        #
+        # try:
+        #     import pickle5 as pickle
+        # except Exception:
+        #     import pickle  # type: ignore
+        #
+        # pickled = pickle.dumps(data, protocol=5)
+        # _data = {"python_object": {"type": "chunk", "chunk": pickled, "codec": "raw"}}
+        #
+        # serialized_data = {
+        #     "data_type": self.data_type_name,
+        #     "data_type_config": self.type_config.dict(),
+        #     "data": _data,
+        #     "serialization_profile": "pickle",
+        #     "serialization_metadata": {
+        #         "profile": "pickle",
+        #         "environment": {},
+        #         "deserialize": {
+        #             "object": {
+        #                 "module_name": "value.unpickle",
+        #                 "module_config": {
+        #                     "value_type": "any",
+        #                     "target_profile": "object",
+        #                     "serialization_profile": "pickle",
+        #                 },
+        #             }
+        #         },
+        #     },
+        # }
+        # from kiara.models.values.value import SerializationResult
+        #
+        # serialized = SerializationResult(**serialized_data)
+        # return serialized
 
     @property
     def type_config(self) -> TYPE_CONFIG_CLS:
@@ -228,135 +329,103 @@ class DataType(abc.ABC, Generic[TYPE_PYTHON_CLS, TYPE_CONFIG_CLS]):
 
     def _pre_examine_data(
         self, data: Any, schema: ValueSchema
-    ) -> Tuple[Any, ValueStatus, int]:
+    ) -> Tuple[Any, Union[str, "SerializedData"], ValueStatus, str, int]:
 
         if data == SpecialValue.NOT_SET:
             status = ValueStatus.NOT_SET
+            data = None
         else:
             status = ValueStatus.SET
 
-            if data is None:
-                if schema.default in [None, SpecialValue.NO_VALUE]:
-                    data = SpecialValue.NO_VALUE
-                    status = ValueStatus.NONE
-                elif schema.default == SpecialValue.NOT_SET:
-                    data = SpecialValue.NOT_SET
-                    status = ValueStatus.NOT_SET
-                elif callable(schema.default):
-                    data = schema.default()
-                    status = ValueStatus.DEFAULT
-                else:
-                    data = copy.deepcopy(schema.default)
-                    status = ValueStatus.DEFAULT
+        if data is None and schema.default not in [
+            None,
+            SpecialValue.NO_VALUE,
+            SpecialValue.NOT_SET,
+        ]:
+
+            status = ValueStatus.DEFAULT
+            if callable(schema.default):
+                data = schema.default()
+            else:
+                data = copy.deepcopy(schema.default)
+
+        if data in [None, SpecialValue.NO_VALUE, SpecialValue.NOT_SET]:
+            if schema.default in [None, SpecialValue.NO_VALUE]:
+                data = SpecialValue.NO_VALUE
+                status = ValueStatus.NONE
+            elif schema.default == SpecialValue.NOT_SET:
+                data = SpecialValue.NOT_SET
+                status = ValueStatus.NOT_SET
+
+            size = 0
+            value_hash = INVALID_HASH_MARKER
+            serialized: Union[None, str, "SerializedData"] = NO_SERIALIZATION_MARKER
+        else:
+
+            from kiara.models.values.value import SerializedData
+
+            if isinstance(data, SerializedData):
+                # TODO: assert value is in schema lineage
+                # assert data.data_type == schema.type
+                # assert data.data_type_config == schema.type_config
+                serialized = data
+                not_serialized: bool = False
             else:
                 data = self.parse_python_obj(data)
                 if data is None:
                     raise Exception(
                         f"Invalid data, can't parse into a value of type '{schema.type}'."
                     )
+                self._validate(data)
 
-        if status in [ValueStatus.SET, ValueStatus.DEFAULT]:
-            value_hash = self.calculate_hash(data)
-        else:
-            value_hash = 0
+                serialized = self.serialize(data)
+                if serialized is None:
+                    serialized = NO_SERIALIZATION_MARKER
 
-        return (data, status, value_hash)
+                if isinstance(serialized, str):
+                    not_serialized = True
+                else:
+                    not_serialized = False
 
-    # def create_value(self, data: Any, schema: Optional[ValueSchema]=None, pedigree: Optional[ValuePedigree]=None) -> TYPE_VALUE_CLS:
-    #
-    #     if schema is None:
-    #         raise NotImplementedError()
-    #
-    #     if pedigree is None:
-    #         raise NotImplementedError()
-    #
-    #     data, status, value_hash = self._pre_examine_data(data=data, schema=schema)
-    #
-    #     v_id = uuid.uuid4()
-    #     value = self.assemble_value(value_id=v_id, data=data, schema=schema, status=status, value_hash=value_hash, pedigree=pedigree, kiara_id=kiara_id)
-    #     return value
+            if not_serialized:
+                size = INVALID_SIZE_MARKER
+                value_hash = INVALID_HASH_MARKER
+            else:
+                size = serialized.data_size  # type: ignore
+                value_hash = serialized.instance_id  # type: ignore
 
-    # def reassemble_value(self, value_id: uuid.UUID, load_config: "LoadConfig", schema: ValueSchema, status: Union[ValueStatus, str],
-    #                        value_hash: int, value_size: int, pedigree: ValuePedigree, kiara_id: uuid.UUID,
-    #                        pedigree_output_name: str) -> TYPE_VALUE_CLS:
-    #
-    #     if isinstance(status, str):
-    #         status = ValueStatus(status)
-    #
-    #     value_cls = self.value_class()
-    #     if status in [ValueStatus.SET, ValueStatus.DEFAULT]:
-    #
-    #         try:
-    #
-    #             value = value_cls(
-    #                 value_id=value_id,
-    #                 kiara_id=kiara_id,
-    #                 value_status=status,
-    #                 value_size=value_size,
-    #                 value_hash=value_hash,
-    #                 value_schema=schema,
-    #                 pedigree=pedigree,
-    #                 pedigree_output_name=pedigree_output_name
-    #             )
-    #
-    #         except Exception as e:
-    #             raise KiaraValueException(
-    #                 data_type=self.__class__, value_data=load_config, exception=e
-    #             )
-    #         retriever = LoadConfigRetriever(load_config=load_config)
-    #         value._data_retriever = retriever
-    #
-    #     else:
-    #         assert value_size == 0
-    #         value = value_cls(
-    #             value_id=value_id,
-    #             kiara_id=kiara_id,
-    #             value_status=status,
-    #             value_size=value_size,
-    #             value_hash=value_hash,
-    #             value_schema=schema,
-    #             pedigree=pedigree,
-    #             pedigree_output_name=pedigree_output_name
-    #         )
-    #
-    #         if status == ValueStatus.NONE:
-    #             data = SpecialValue.NO_VALUE
-    #         else:
-    #             data = SpecialValue.NOT_SET
-    #
-    #         retriever = StaticDataRetriever(data=data)
-    #         value._data_retriever = retriever
-    #
-    #     return value
+        assert serialized is not None
+        result = (data, serialized, status, value_hash, size)
+        return result
 
     def assemble_value(
         self,
         value_id: uuid.UUID,
         data: Any,
         schema: ValueSchema,
+        serialized: Union[str, "SerializedData"],
         status: Union[ValueStatus, str],
-        value_hash: int,
-        pedigree: ValuePedigree,
+        value_hash: str,
+        value_size: int,
+        pedigree: "ValuePedigree",
         kiara_id: uuid.UUID,
         pedigree_output_name: str,
-    ) -> Tuple[Value, Any]:
+    ) -> Tuple["Value", Any]:
+
+        from kiara.models.values.value import Value
 
         if isinstance(status, str):
             status = ValueStatus(status).name
 
         this_cls = PythonClass.from_class(self.__class__)
         if status in [ValueStatus.SET, ValueStatus.DEFAULT]:
-            size = self.calculate_size(data)
-
             try:
 
-                self._validate(data)
-
                 value = Value(
                     value_id=value_id,
                     kiara_id=kiara_id,
                     value_status=status,
-                    value_size=size,
+                    value_size=value_size,
                     value_hash=value_hash,
                     value_schema=schema,
                     pedigree=pedigree,
@@ -369,12 +438,11 @@ class DataType(abc.ABC, Generic[TYPE_PYTHON_CLS, TYPE_CONFIG_CLS]):
                     data_type=self.__class__, value_data=data, exception=e
                 )
         else:
-            size = 0
             value = Value(
                 value_id=value_id,
                 kiara_id=kiara_id,
                 value_status=status,
-                value_size=size,
+                value_size=value_size,
                 value_hash=value_hash,
                 value_schema=schema,
                 pedigree=pedigree,
@@ -384,6 +452,7 @@ class DataType(abc.ABC, Generic[TYPE_PYTHON_CLS, TYPE_CONFIG_CLS]):
 
         value._value_data = data
         value._data_type = self
+        value._serialized_data = serialized
         return value, data
 
     def parse_python_obj(self, data: Any) -> TYPE_PYTHON_CLS:
@@ -410,11 +479,29 @@ class DataType(abc.ABC, Generic[TYPE_PYTHON_CLS, TYPE_CONFIG_CLS]):
                 f"Invalid python type '{type(value)}', must be: {self.__class__.python_class()}"
             )
 
-    def __rich_console__(
-        self, console: Console, options: ConsoleOptions
-    ) -> RenderResult:
+    def create_renderable(self, **config):
 
-        raise NotImplementedError()
+        show_type_info = config.get("show_type_info", False)
+
+        table = Table(show_header=False, box=box.SIMPLE)
+        table.add_column("key")
+        table.add_column("value", style="i")
+        table.add_row("type_name", self.data_type_name)
+        config_json = self.type_config.json(
+            exclude_unset=True, option=orjson.OPT_INDENT_2
+        )
+        config = Syntax(config_json, "json", background_color="default")
+        table.add_row("type_config", config)
+
+        if show_type_info:
+            from kiara.models.values.data_type import DataTypeClassInfo
+
+            info = DataTypeClassInfo.create_from_type_class(self.__class__)
+            table.add_row("", "")
+            table.add_row("", Rule())
+            table.add_row("type_info", info)
+
+        return table
 
 
 # class ValueTypeInfo(object):
