@@ -11,6 +11,7 @@ import sys
 import typing
 
 from kiara import Kiara
+from kiara.interfaces.python_api.operation import KiaraOperation
 from kiara.models.module.operation import OperationGroupInfo, OperationTypeClassesInfo
 from kiara.utils.cli import output_format_option, terminal_print_model
 
@@ -75,8 +76,8 @@ def list_types(ctx, full_doc, format: str, filter: typing.Iterable[str]):
     help="Display the full doc for all operations (when using 'terminal' as format).",
 )
 @click.option(
-    "--include-internal-operations",
-    "-i",
+    "--include-internal",
+    "-I",
     help="Whether to include operations that are mainly used internally.",
     is_flag=True,
 )
@@ -87,7 +88,7 @@ def list_operations(
     by_type: bool,
     filter: typing.Iterable[str],
     full_doc: bool,
-    include_internal_operations: bool,
+    include_internal: bool,
     format: str,
 ):
 
@@ -108,7 +109,7 @@ def list_operations(
                 temp[op_id] = op
         operations = temp
 
-    if not include_internal_operations:
+    if not include_internal:
         temp = {}
         for op_id, op in operations.items():
             if not op.operation_details.is_internal_operation:
@@ -124,7 +125,7 @@ def list_operations(
         ops_info,
         format=format,
         in_panel=title,
-        include_internal_operations=include_internal_operations,
+        include_internal_operations=include_internal,
         full_doc=full_doc,
         by_type=by_type,
     )
@@ -138,14 +139,18 @@ def list_operations(
     help="Show module source code (or pipeline configuration).",
     is_flag=True,
 )
+@click.option(
+    "--module-info", "-m", help="Show module type and config information.", is_flag=True
+)
 @output_format_option()
 @click.pass_context
-def explain(ctx, operation_id: str, source: bool, format: str):
+def explain(ctx, operation_id: str, source: bool, format: str, module_info: bool):
 
     kiara_obj: Kiara = ctx.obj["kiara"]
 
     if os.path.isfile(os.path.realpath(operation_id)):
-        raise NotImplementedError()
+        kiara_op = KiaraOperation(kiara=kiara_obj, operation_name=operation_id)
+        op_config = kiara_op.operation
     else:
         op_config = kiara_obj.operation_registry.get_operation(operation_id)
 
@@ -159,4 +164,5 @@ def explain(ctx, operation_id: str, source: bool, format: str):
         format=format,
         in_panel=f"Operation: [b i]{operation_id}[/b i]",
         include_src=source,
+        include_module_details=module_info,
     )
