@@ -197,7 +197,7 @@ def run(
     # prepare inputs
     list_keys = []
     for name, value_schema in operation.operation_details.inputs_schema.items():
-        if value_schema.type in ["array", "list"]:
+        if value_schema.type in ["list"]:
             list_keys.append(name)
 
     inputs_dict = dict_from_cli_args(*inputs, list_keys=list_keys)
@@ -272,9 +272,8 @@ def run(
     if save:
 
         save_results = kiara_op.save_result(job_id=job_id, aliases=final_aliases)
-        if not silent:
-            if len(save_results) == 1:
-                title = "[b]Stored result value[/b]"
-            else:
-                title = "[b]Stored result values[/b]"
-            terminal_print(save_results, in_panel=title, empty_line_before=True)
+        if len(save_results) == 1:
+            title = "[b]Stored result value[/b]"
+        else:
+            title = "[b]Stored result values[/b]"
+        terminal_print(save_results, in_panel=title, empty_line_before=True)
