@@ -6,29 +6,40 @@
 #  Mozilla Public License, version 2.0 (see LICENSE or https://www.mozilla.org/en-US/MPL/2.0/)
 
 import datetime
-import hashlib
 import os
 import shutil
 import structlog
 from deepdiff import DeepHash
-from pydantic import BaseModel, Field, PrivateAttr, validator
+from multiformats import CID
+from pydantic import BaseModel, Field, PrivateAttr
+from rich import box
+from rich.console import RenderableType
+from rich.table import Table
 from typing import Any, Dict, List, Mapping, Optional, Union
 
-from kiara.defaults import (
-    DEFAULT_EXCLUDE_FILES,
-    FILE_BUNDLE_MODEL_CATEOGORY_ID,
-    FILE_MODEL_CATEOGORY_ID,
-    KIARA_HASH_FUNCTION,
-)
+from kiara.defaults import DEFAULT_EXCLUDE_FILES, KIARA_HASH_FUNCTION
 from kiara.models import KiaraModel
 from kiara.utils import log_message
+from kiara.utils.hashing import compute_cid_from_file
 
 logger = structlog.getLogger()
 
+FILE_BUNDLE_IMPORT_AVAILABLE_COLUMNS = [
+    "id",
+    "rel_path",
+    "import_time",
+    "mime_type",
+    "size",
+    "content",
+    "file_name",
+]
+
 
 class FileModel(KiaraModel):
     """Describes properties for the 'file' value type."""
 
+    _kiara_model_id = "instance.data.file"
+
     @classmethod
     def load_file(
         cls,
@@ -77,38 +88,40 @@ class FileModel(KiaraModel):
             mime_type=mime_type,
             size=size,
             file_name=file_name,
-            path=path,
         )
+        m._path = path
         return m
 
-    _file_hash: Optional[str] = PrivateAttr(default=None)
-
     import_time: datetime.datetime = Field(
         description="The time when the file was imported."
     )
     mime_type: str = Field(description="The mime type of the file.")
     file_name: str = Field("The name of the file.")
     size: int = Field(description="The size of the file.")
-    path: str = Field(description="The path to the file.")
 
-    @validator("path")
-    def ensure_abs_path(cls, value):
-        return os.path.abspath(value)
+    _path: Optional[str] = PrivateAttr(default=None)
+    _file_hash: Optional[str] = PrivateAttr(default=None)
+    _file_cid: Optional[CID] = PrivateAttr(default=None)
 
-    def _retrieve_id(self) -> str:
-        return self.path
+    # @validator("path")
+    # def ensure_abs_path(cls, value):
+    #     return os.path.abspath(value)
 
-    def _retrieve_category_id(self) -> str:
-        return FILE_MODEL_CATEOGORY_ID
+    @property
+    def path(self) -> str:
+        if self._path is None:
+            raise Exception("File path not set for file model.")
+        return self._path
 
     def _retrieve_data_to_hash(self) -> Any:
-        return {
+        data = {
             "file_name": self.file_name,
-            "file_hash": self.file_hash,
+            "file_cid": self.file_cid,
         }
+        return data
 
-    def get_id(self) -> str:
-        return self.path
+    # def get_id(self) -> str:
+    #     return self.path
 
     def get_category_alias(self) -> str:
         return "instance.file_model"
@@ -134,15 +147,19 @@ class FileModel(KiaraModel):
         if self._file_hash is not None:
             return self._file_hash
 
-        sha256_hash = hashlib.sha3_256()
-        with open(self.path, "rb") as f:
-            # Read and update hash string value in blocks of 4K
-            for byte_block in iter(lambda: f.read(4096), b""):
-                sha256_hash.update(byte_block)
-
-        self._file_hash = sha256_hash.hexdigest()
+        self._file_hash = str(self.file_cid)
         return self._file_hash
 
+    @property
+    def file_cid(self) -> CID:
+
+        if self._file_cid is not None:
+            return self._file_cid
+
+        # TODO: auto-set codec?
+        self._file_cid = compute_cid_from_file(file=self.path, codec="raw")
+        return self._file_cid
+
     @property
     def file_name_without_extension(self) -> str:
 
@@ -194,13 +211,16 @@ class FolderImportConfig(BaseModel):
 class FileBundle(KiaraModel):
     """Describes properties for the 'file_bundle' value type."""
 
+    _kiara_model_id = "instance.data.file_bundle"
+
     @classmethod
     def import_folder(
         cls,
         source: str,
+        bundle_name: Optional[str] = None,
         import_config: Union[None, Mapping[str, Any], FolderImportConfig] = None,
         import_time: Optional[datetime.datetime] = None,
-    ):
+    ) -> "FileBundle":
 
         if not source:
             raise ValueError("No source path provided.")
@@ -214,7 +234,7 @@ class FileBundle(KiaraModel):
         if source.endswith(os.path.sep):
             source = source[0:-1]
 
-        path = os.path.abspath(source)
+        abs_path = os.path.abspath(source)
 
         if import_config is None:
             _import_config = FolderImportConfig()
@@ -251,7 +271,7 @@ class FileBundle(KiaraModel):
             else:
                 return any(filename.endswith(ext) for ext in valid_extensions)
 
-        for root, dirnames, filenames in os.walk(path, topdown=True):
+        for root, dirnames, filenames in os.walk(abs_path, topdown=True):
 
             if exclude_dirs:
                 dirnames[:] = [d for d in dirnames if d not in exclude_dirs]
@@ -263,7 +283,7 @@ class FileBundle(KiaraModel):
             ]:
 
                 full_path = os.path.join(root, filename)
-                rel_path = os.path.relpath(full_path, path)
+                rel_path = os.path.relpath(full_path, abs_path)
 
                 file_model = FileModel.load_file(
                     full_path, import_time=bundle_import_time
@@ -271,25 +291,27 @@ class FileBundle(KiaraModel):
                 sum_size = sum_size + file_model.size
                 included_files[rel_path] = file_model
 
-        bundle_name = os.path.basename(source)
+        if bundle_name is None:
+            bundle_name = os.path.basename(source)
 
-        return FileBundle.create_from_file_models(
+        bundle = FileBundle.create_from_file_models(
             files=included_files,
-            path=path,
+            path=abs_path,
             bundle_name=bundle_name,
             sum_size=sum_size,
             import_time=bundle_import_time,
         )
+        return bundle
 
     @classmethod
     def create_from_file_models(
-        self,
+        cls,
         files: Mapping[str, FileModel],
         bundle_name: str,
-        path: str,
+        path: Optional[str] = None,
         sum_size: Optional[int] = None,
         import_time: Optional[datetime.datetime] = None,
-    ):
+    ) -> "FileBundle":
 
         if import_time:
             bundle_import_time = import_time
@@ -300,7 +322,6 @@ class FileBundle(KiaraModel):
 
         result["included_files"] = files
 
-        result["path"] = path
         result["import_time"] = datetime.datetime.now().isoformat()
         result["number_of_files"] = len(files)
         result["bundle_name"] = bundle_name
@@ -312,11 +333,12 @@ class FileBundle(KiaraModel):
                 sum_size = sum_size + f.size
         result["size"] = sum_size
 
-        return FileBundle(**result)
+        bundle = FileBundle(**result)
+        bundle._path = path
+        return bundle
 
     _file_bundle_hash: Optional[int] = PrivateAttr(default=None)
 
-    path: str = Field(description="The archive path of the folder.")
     bundle_name: str = Field(description="The name of this bundle.")
     import_time: datetime.datetime = Field(
         description="The time when the file bundle was imported."
@@ -328,19 +350,32 @@ class FileBundle(KiaraModel):
         description="A map of all the included files, incl. their properties. Uses the relative path of each file as key."
     )
     size: int = Field(description="The size of all files in this folder, combined.")
-
-    def _retrieve_id(self) -> str:
-        return self.path
-
-    def _retrieve_category_id(self) -> str:
-        return FILE_BUNDLE_MODEL_CATEOGORY_ID
+    _path: Optional[str] = PrivateAttr(default=None)
 
     @property
-    def model_data_hash(self) -> int:
-        return self.file_bundle_hash
+    def path(self) -> str:
+        if self._path is None:
+            # TODO: better explanation, offer remedy like copying into temp folder
+            raise Exception(
+                "File bundle path not set, it appears this bundle is comprised of symlinks only."
+            )
+        return self._path
+
+    def _retrieve_id(self) -> str:
+        return str(self.file_bundle_hash)
+
+    # @property
+    # def model_data_hash(self) -> int:
+    #     return self.file_bundle_hash
 
     def _retrieve_data_to_hash(self) -> Any:
-        raise NotImplementedError()
+
+        return {
+            "bundle_name": self.bundle_name,
+            "included_files": {
+                k: v.instance_cid for k, v in self.included_files.items()
+            },
+        }
 
     def get_relative_path(self, file: FileModel):
         return os.path.relpath(file.path, self.path)
@@ -349,22 +384,25 @@ class FileBundle(KiaraModel):
 
         content_dict: Dict[str, str] = {}
 
-        def read_file(rel_path: str, fm: FileModel):
-            with open(fm.path, encoding="utf-8") as f:
+        def read_file(rel_path: str, full_path: str):
+            with open(full_path, encoding="utf-8") as f:
                 try:
                     content = f.read()
                     content_dict[rel_path] = content  # type: ignore
                 except Exception as e:
                     if ignore_errors:
                         log_message(f"Can't read file: {e}")
-                        logger.warning("ignore.file", path=fm.path, reason=str(e))
+                        logger.warning("ignore.file", path=full_path, reason=str(e))
                     else:
-                        raise Exception(f"Can't read file (as text) '{fm.path}: {e}")
+                        raise Exception(f"Can't read file (as text) '{full_path}: {e}")
 
         # TODO: common ignore files and folders
-        for f in self.included_files.values():
-            rel_path = self.get_relative_path(f)
-            read_file(rel_path=rel_path, fm=f)
+        for rel_path, f in self.included_files.items():
+            if f._path:
+                path = f._path
+            else:
+                path = self.get_relative_path(f)
+            read_file(rel_path=rel_path, full_path=path)
 
         return content_dict
 
@@ -409,6 +447,62 @@ class FileBundle(KiaraModel):
 
         return fb
 
+    def create_renderable(self, **config: Any) -> RenderableType:
+
+        show_bundle_hash = config.get("show_bundle_hash", False)
+
+        table = Table(show_header=False, box=box.SIMPLE)
+        table.add_column("key")
+        table.add_column("value", style="i")
+
+        table.add_row("bundle name", self.bundle_name)
+        table.add_row("import_time", str(self.import_time))
+        table.add_row("number_of_files", str(self.number_of_files))
+        table.add_row("size", str(self.size))
+        if show_bundle_hash:
+            table.add_row("bundle_hash", str(self.file_bundle_hash))
+
+        content = self._create_content_table(**config)
+        table.add_row("included files", content)
+
+        return table
+
+    def _create_content_table(self, **render_config: Any) -> Table:
+
+        # show_content = render_config.get("show_content_preview", False)
+        max_no_included_files = render_config.get("max_no_files", 40)
+
+        table = Table(show_header=True, box=box.SIMPLE)
+        table.add_column("(relative) path")
+        table.add_column("size")
+        # if show_content:
+        #     table.add_column("content preview")
+
+        if (
+            max_no_included_files < 0
+            or len(self.included_files) <= max_no_included_files
+        ):
+            for f, model in self.included_files.items():
+                row = [f, str(model.size)]
+                table.add_row(*row)
+        else:
+            files = list(self.included_files.keys())
+            half = int((max_no_included_files - 1) / 2)
+            head = files[0:half]
+            tail = files[-1 * half :]  # noqa
+            for rel_path in head:
+                model = self.included_files[rel_path]
+                row = [rel_path, str(model.size)]
+                table.add_row(*row)
+            table.add_row("   ... output skipped ...", "")
+            table.add_row("   ... output skipped ...", "")
+            for rel_path in tail:
+                model = self.included_files[rel_path]
+                row = [rel_path, str(model.size)]
+                table.add_row(*row)
+
+        return table
+
     def __repr__(self):
         return f"FileBundle(name={self.bundle_name})"
 
