@@ -4,7 +4,7 @@
 #  Copyright (c) 2021, Markus Binsteiner
 #
 #  Mozilla Public License, version 2.0 (see LICENSE or https://www.mozilla.org/en-US/MPL/2.0/)
-
+import humanfriendly
 import orjson
 import uuid
 from pydantic import BaseModel, Field, PrivateAttr
@@ -78,6 +78,10 @@ RENDER_FIELDS: Dict[str, Dict[str, Any]] = {
             "terminal": lambda p: p.property_values.create_renderable(show_header=False)
         },
     },
+    "size": {
+        "show_default": True,
+        "render": {"terminal": lambda v: humanfriendly.format_size(v.value_size)},
+    },
 }
 
 
@@ -216,7 +220,7 @@ class ValuesInfo(BaseModel):
     def create_render_map(self, render_type: str = "terminal", **render_config):
 
         list_by_alias = render_config.get("list_by_alias", True)
-        show_internal = render_config.get("show_internal", False)
+        show_internal = render_config.get("show_internal_values", False)
 
         render_fields = render_config.get("render_fields", None)
         if not render_fields:
@@ -284,6 +288,8 @@ class ValuesInfo(BaseModel):
         for property in render_fields:
             if property == "aliases" and list_by_alias:
                 table.add_column("alias")
+            elif property == "size":
+                table.add_column("size", justify="right")
             else:
                 table.add_column(property)
 
