@@ -78,6 +78,7 @@ class InputOutputObject(abc.ABC):
 
         self._alias: str = alias
         self._inputs_schema: Mapping[str, ValueSchema] = None  # type: ignore
+        self._full_inputs_schema: Mapping[str, ValueSchema] = None  # type: ignore
         self._outputs_schema: Mapping[str, ValueSchema] = None  # type: ignore
         self._constants: Mapping[str, ValueSchema] = None  # type: ignore
 
@@ -128,6 +129,16 @@ class InputOutputObject(abc.ABC):
 
         return self._inputs_schema  # type: ignore
 
+    @property
+    def full_inputs_schema(self) -> Mapping[str, ValueSchema]:
+
+        if self._full_inputs_schema is not None:
+            return self._full_inputs_schema
+
+        self._full_inputs_schema = dict(self.inputs_schema)
+        self._full_inputs_schema.update(self._constants)
+        return self._full_inputs_schema
+
     @property
     def constants(self) -> Mapping[str, ValueSchema]:
 
@@ -222,6 +233,7 @@ class InputOutputObject(abc.ABC):
         return self.outputs_schema.keys()
 
     def augment_module_inputs(self, inputs: Mapping[str, Any]) -> Dict[str, Any]:
+
         return augment_values(
             values=inputs, schemas=self.inputs_schema, constants=self.constants
         )
@@ -231,6 +243,8 @@ class InputOutputObject(abc.ABC):
 
 
 class ModuleCharacteristics(BaseModel):
+    class Config:
+        allow_mutation = False
 
     is_idempotent: bool = Field(
         description="Whether this module is idempotent (aka always produces the same output with the same inputs.",
@@ -242,6 +256,11 @@ class ModuleCharacteristics(BaseModel):
     )
 
 
+DEFAULT_IDEMPOTENT_MODULE_CHARACTERISTICS = ModuleCharacteristics(
+    is_idempotent=True, is_internal=False
+)
+
+
 class KiaraModule(InputOutputObject, Generic[KIARA_CONFIG]):
     """The base class that every custom module in *Kiara* needs to inherit from.
 
@@ -362,7 +381,8 @@ class KiaraModule(InputOutputObject, Generic[KIARA_CONFIG]):
         return self._characteristics
 
     def _retrieve_module_characteristics(self) -> ModuleCharacteristics:
-        return ModuleCharacteristics()
+
+        return DEFAULT_IDEMPOTENT_MODULE_CHARACTERISTICS
 
     def get_config_value(self, key: str) -> Any:
         """Retrieve the value for a specific configuration option.
