@@ -1,8 +1,15 @@
 # -*- coding: utf-8 -*-
-from kiara.data.values.value_set import ValueSet
-from kiara.module import KiaraModule
-from kiara.processing import JobLog, JobStatus
-from kiara.processing.processor import ModuleProcessor, ProcessorConfig
+
+#  Copyright (c) 2021, University of Luxembourg / DHARPA project
+#  Copyright (c) 2021, Markus Binsteiner
+#
+#  Mozilla Public License, version 2.0 (see LICENSE or https://www.mozilla.org/en-US/MPL/2.0/)
+
+import uuid
+
+from kiara.models.values.value import ValueMap, ValueMapWritable
+from kiara.modules import KiaraModule
+from kiara.processing import JobLog, JobStatus, ModuleProcessor, ProcessorConfig
 
 try:
     pass
@@ -16,12 +23,12 @@ class SynchronousProcessorConfig(ProcessorConfig):
 
 
 class SynchronousProcessor(ModuleProcessor):
-    def process(
+    def _add_processing_task(
         self,
-        job_id: str,
-        module: KiaraModule,
-        inputs: ValueSet,
-        outputs: ValueSet,
+        job_id: uuid.UUID,
+        module: "KiaraModule",
+        inputs: ValueMap,
+        outputs: ValueMapWritable,
         job_log: JobLog,
     ):
 
@@ -33,7 +40,7 @@ class SynchronousProcessor(ModuleProcessor):
         except Exception as e:
             self.job_status_updated(job_id=job_id, status=e)
 
-    def _wait_for(self, *job_ids: str):
+    def _wait_for(self, *job_ids: uuid.UUID):
 
         # jobs will always be finished, since we were waiting for them in the 'process' method
         return
