@@ -33,7 +33,12 @@ from kiara.defaults import (
     STRICT_CHECKS,
     SpecialValue,
 )
-from kiara.exceptions import InvalidValuesException
+from kiara.exceptions import (
+    InvalidValuesException,
+    NoSuchValueAliasException,
+    NoSuchValueException,
+    NoSuchValueIdException,
+)
 from kiara.models.events.data_registry import (
     DataArchiveAddedEvent,
     ValueCreatedEvent,
@@ -168,12 +173,15 @@ class DataRegistry(object):
         set_as_default_store: Optional[bool] = None,
     ):
 
-        data_store_id = archive.register_archive(kiara=self._kiara)
+        data_store_id = archive.archive_id
+        archive.register_archive(kiara=self._kiara)
         if alias is None:
             alias = str(data_store_id)
 
         if alias in self._data_archives.keys():
-            raise Exception(f"Can't add store, alias '{alias}' already registered.")
+            raise Exception(
+                f"Can't add data archive, alias '{alias}' already registered."
+            )
         self._data_archives[alias] = archive
         is_store = False
         is_default_store = False
@@ -267,6 +275,7 @@ class DataRegistry(object):
 
     def get_value(self, value_id: Union[uuid.UUID, Value, str]) -> Value:
         _value_id = None
+
         if not isinstance(value_id, uuid.UUID):
             # fallbacks for common mistakes, this should error out if not a Value or string.
             if hasattr(value_id, "value_id"):
@@ -301,8 +310,9 @@ class DataRegistry(object):
                             alias=rest
                         )
                         if _value_id is None:
-                            raise Exception(
-                                f"Can't retrive value for alias '{rest}': no such alias registered."
+                            raise NoSuchValueAliasException(
+                                alias=rest,
+                                msg=f"Can't retrive value for alias '{rest}': no such alias registered.",
                             )
                     else:
                         raise Exception(
@@ -324,10 +334,13 @@ class DataRegistry(object):
                 matches.append(store_id)
 
         if len(matches) == 0:
-            raise Exception(f"No value registered with id: {value_id}")
+            raise NoSuchValueIdException(
+                value_id=_value_id, msg=f"No value registered with id: {value_id}"
+            )
         elif len(matches) > 1:
-            raise Exception(
-                f"Found value with id '{value_id}' in multiple archives, this is not supported (yet): {matches}"
+            raise NoSuchValueIdException(
+                value_id=_value_id,
+                msg=f"Found value with id '{value_id}' in multiple archives, this is not supported (yet): {matches}",
             )
 
         self._value_archive_lookup_map[_value_id] = matches[0]
@@ -512,6 +525,8 @@ class DataRegistry(object):
                 value.value_hash,
                 value.value_size,
             )
+        except NoSuchValueException as nsve:
+            raise nsve
         except Exception:
             # TODO: differentiate between 'value not found' and other type of errors
             pass
@@ -794,9 +809,14 @@ class DataRegistry(object):
 
         result = op.run(kiara=self._kiara, inputs=inputs)
         python_object = result.get_value_data(result_field_match)
-        self._cached_data[value.value_id] = python_object
 
-        return python_object
+        # TODO: can we do without this?
+        parsed = value.data_type.parse_python_obj(python_object)
+        value.data_type._validate(parsed)
+
+        self._cached_data[value.value_id] = parsed
+
+        return parsed
 
         # op_type: DeSerializeOperationType = self._kiara.operation_registry.get_operation_type("deserialize")  # type: ignore
         # ops = op_type.find_deserialzation_operation_for_type_and_profile(
