@@ -5,6 +5,7 @@
 #
 #  Mozilla Public License, version 2.0 (see LICENSE or https://www.mozilla.org/en-US/MPL/2.0/)
 import orjson
+import shutil
 import structlog
 import uuid
 from enum import Enum
@@ -21,7 +22,7 @@ from kiara.models.values.value import (
     SerializedData,
     Value,
 )
-from kiara.registries import FileSystemArchiveConfig
+from kiara.registries import ArchiveDetails, FileSystemArchiveConfig
 from kiara.registries.data.data_store import BaseDataStore, DataArchive
 from kiara.registries.ids import ID_REGISTRY
 from kiara.registries.jobs import JobArchive
@@ -75,16 +76,26 @@ class FileSystemDataArchive(DataArchive, JobArchive):
     # def get_job_archive_id(self) -> uuid.UUID:
     #     return self._kiara.id
 
+    def get_archive_details(self) -> ArchiveDetails:
+
+        size = sum(
+            f.stat().st_size for f in self.data_store_path.glob("**/*") if f.is_file()
+        )
+        return ArchiveDetails(size=size)
+
     @property
     def data_store_path(self) -> Path:
 
         if self._base_path is not None:
             return self._base_path
 
-        self._base_path = Path(self.config.archive_path).absolute()
+        self._base_path = Path(self.config.archive_path).absolute()  # type: ignore
         self._base_path.mkdir(parents=True, exist_ok=True)
         return self._base_path
 
+    def _delete_archive(self):
+        shutil.rmtree(self.data_store_path)
+
     @property
     def hash_fs_path(self) -> Path:
 
