@@ -4,15 +4,15 @@
 #  Copyright (c) 2021, Markus Binsteiner
 #
 #  Mozilla Public License, version 2.0 (see LICENSE or https://www.mozilla.org/en-US/MPL/2.0/)
-
 import orjson
+import shutil
 import uuid
 from pathlib import Path
 from typing import Any, Dict, Iterable, Optional
 
 from kiara.models.module.jobs import JobRecord
 from kiara.models.module.manifest import InputsManifest
-from kiara.registries import FileSystemArchiveConfig
+from kiara.registries import ArchiveDetails, FileSystemArchiveConfig
 from kiara.registries.jobs import MANIFEST_SUB_PATH, JobArchive, JobStore
 
 
@@ -34,16 +34,27 @@ class FileSystemJobArchive(JobArchive):
         super().__init__(archive_id=archive_id, config=config)
         self._base_path: Optional[Path] = None
 
+    def get_archive_details(self) -> ArchiveDetails:
+
+        size = sum(
+            f.stat().st_size for f in self.job_store_path.glob("**/*") if f.is_file()
+        )
+        return ArchiveDetails(size=size)
+
     @property
     def job_store_path(self) -> Path:
 
         if self._base_path is not None:
             return self._base_path
 
-        self._base_path = Path(self.config.archive_path).absolute()
+        self._base_path = Path(self.config.archive_path).absolute()  # type: ignore
         self._base_path.mkdir(parents=True, exist_ok=True)
         return self._base_path
 
+    def _delete_archive(self):
+
+        shutil.rmtree(self.job_store_path)
+
     def find_matching_job_record(
         self, inputs_manifest: InputsManifest
     ) -> Optional[JobRecord]:
