@@ -72,7 +72,7 @@ class ModuleRegistry(object):
             else:
                 result[type_name] = md
 
-        return ModuleTypeClassesInfo.construct(group_alias=alias, type_infos=result)  # type: ignore
+        return ModuleTypeClassesInfo.construct(group_alias=alias, item_infos=result)  # type: ignore
 
     def create_module(self, manifest: Union[Manifest, str]) -> "KiaraModule":
         """Create a [KiaraModule][kiara.module.KiaraModule] object from a module configuration.
