@@ -1,9 +1,14 @@
 # -*- coding: utf-8 -*-
+
+#  Copyright (c) 2021, University of Luxembourg / DHARPA project
+#
+#  Mozilla Public License, version 2.0 (see LICENSE or https://www.mozilla.org/en-US/MPL/2.0/)
+
 import typing
 from inspect import cleandoc
 
 from kiara.defaults import DEFAULT_NO_DESC_VALUE
-from kiara.utils.output import first_line
+from kiara.utils import first_line
 
 
 def extract_doc_from_cls(cls: typing.Type, only_first_line: bool = False):
